import React from "react";
import { DatePickerCalendar } from "../date-picker-calendar";
import { DayPickerRangeController } from "../react-dates-wrapper";
import { POSITIONS } from "@orbit-ui/react-popup";
import { PRESET_SHAPE } from "./presets";
import { PureComponent, cloneElement } from "react";
import { START_DATE } from "react-dates/constants";
import { arrayOf, bool, func, node, number, oneOf, oneOfType, shape, string } from "prop-types";
import { isNil } from "lodash";
import { momentObj as momentType } from "react-moment-proptypes";
import moment from "moment";
export class DateRangePickerCalendar extends PureComponent {
  constructor(...args) {
    super(...args);
    this.state = {
      // Must be non-null in order to select dates.
      focusedInput: START_DATE
    };

    this.handleFocusChange = focusedInput => {
      this.setState({
        focusedInput
      });
    };

    this.handleDatesChange = ({
      startDate,
      endDate
    }) => {
      const {
        onDatesChange
      } = this.props;
      const {
        focusedInput
      } = this.state;

      if (focusedInput === START_DATE) {
        if (!isNil(startDate) && !isNil(endDate)) {
          // By default, when the user select a valid full range then select a date previous to the range, react-dates will extend the current range instead of starting a new one.
          // It works this way because react-dates doesn't reset the endDate.
          onDatesChange(startDate, null, null, this.props);
        } else {
          onDatesChange(startDate, endDate, null, this.props);
        }
      } else {
        // Enable selection of a new single date or range when an end date is selected.
        // This is mostly usefull to allow the selection of a single date after a range has been selected.
        // The default behavior is to select a new end date for the current range.
        if (!isNil(endDate)) {
          this.resetFocusedInput();
        }

        onDatesChange(startDate, endDate, null, this.props);
      }
    };

    this.handleClear = () => {
      const {
        onDatesChange
      } = this.props;
      this.resetFocusedInput();
      onDatesChange(null, null, null, this.props);
    };

    this.handleApply = event => {
      const {
        onApply
      } = this.props;
      this.resetFocusedInput();
      onApply(event, this.props);
    };

    this.handleSelectPreset = (event, preset) => {
      const {
        onDatesChange
      } = this.props;
      this.resetFocusedInput();
      onDatesChange(preset.startDate, preset.endDate, preset.text, this.props);
    };
  }

  getInitialDate() {
    const {
      startDate,
      endDate
    } = this.props;
    return startDate || endDate || moment();
  }

  resetFocusedInput() {
    this.setState({
      focusedInput: START_DATE
    });
  }

  renderPresets() {
    const {
      startDate,
      endDate,
      minDate,
      maxDate,
      presetsComponent,
      presets
    } = this.props;
    return cloneElement(presetsComponent, {
      startDate,
      endDate,
      minDate,
      maxDate,
      onSelectPreset: this.handleSelectPreset,
      presets
    });
  }

  renderButtons() {
    const {
      startDate,
      endDate,
      allowSingleDateSelection,
      allowClear,
      buttons
    } = this.props;
    return cloneElement(buttons, {
      startDate,
      endDate,
      onClear: this.handleClear,
      onApply: this.handleApply,
      allowSingleDateSelection,
      allowClear
    });
  }

  renderCalendar() {
    const {
      startDate,
      endDate,
      allowSingleDateSelection,
      reactDatesCalendar
    } = this.props;
    const {
      focusedInput
    } = this.state;
    return cloneElement(reactDatesCalendar, {
      startDate: startDate,
      endDate: endDate,
      onDatesChange: this.handleDatesChange,
      onFocusChange: this.handleFocusChange,
      focusedInput: focusedInput,
      minimumNights: allowSingleDateSelection ? 0 : 1
    });
  }

  render() {
    const {
      minDate,
      maxDate,
      initialVisibleMonth,
      numberOfMonths,
      position,
      navPrevIcon,
      navNextIcon,
      className
    } = this.props;
    return React.createElement(DatePickerCalendar, {
      calendar: this.renderCalendar(),
      buttons: this.renderButtons(),
      leftContent: this.renderPresets(),
      minDate: minDate,
      maxDate: maxDate,
      initialDate: this.getInitialDate(),
      initialVisibleMonth: initialVisibleMonth,
      numberOfMonths: numberOfMonths,
      position: position,
      navPrevIcon: navPrevIcon,
      navNextIcon: navNextIcon,
      className: className
    });
  }

}
DateRangePickerCalendar.propTypes = {
  startDate: momentType,
  endDate: momentType,
  onDatesChange: func,
  onApply: func,
  allowSingleDateSelection: bool,
  minDate: momentType,
  maxDate: momentType,
  initialVisibleMonth: oneOfType([momentType, func]),
  numberOfMonths: number,
  position: oneOf(POSITIONS),
  navPrevIcon: node,
  navNextIcon: node,
  presetsComponent: node,
  presets: arrayOf(shape(PRESET_SHAPE)),
  buttons: node,
  allowClear: bool,
  reactDatesCalendar: node,
  className: string
};
DateRangePickerCalendar.defaultProps = {
  reactDatesCalendar: React.createElement(DayPickerRangeController, null)
};