import React from "react";
import { ArgumentError, AutoControlledPureComponent, KEYS, getAutoControlledStateFromProps } from "@orbit-ui/react-components-shared";
import { RESULT_SHAPE } from "./results";
import { SearchInputController } from "./search-input-controller";
import { arrayOf, bool, func, number, shape, string } from "prop-types";
import { isNil } from "lodash";
export function startsWithSearch(event, items, query) {
  return items.filter(x => x.text.toUpperCase().startsWith(query.toUpperCase()));
}
export class SearchInput extends AutoControlledPureComponent {
  constructor(...args) {
    super(...args);
    this.state = {
      open: false,
      visibleResults: []
    };

    this.handleSearch = (event, query) => {
      const {
        results,
        onSearch,
        minCharacters
      } = this.props;

      if (query.length >= minCharacters) {
        const newResults = onSearch(event, results, query, this.props);
        this.setState({
          visibleResults: newResults
        });
        this.open(event);
      } else {
        this.setState({
          visibleResults: []
        });
        this.close(event);
      }
    };

    this.handleValueChange = (event, value) => {
      const {
        onValueChange
      } = this.props;
      this.close(event);
      onValueChange(event, value, this.props);
    };

    this.handleClear = event => {
      const {
        results,
        onClear,
        closeOnBlur
      } = this.props;
      this.setState({
        visibleResults: results
      });

      if (!closeOnBlur) {
        this.close(event);
      }

      if (!isNil(onClear)) {
        onClear(event, this.props);
      }
    };

    this.handleBlur = event => {
      const {
        onBlur,
        closeOnBlur
      } = this.props;

      if (closeOnBlur) {
        this.close(event);
      }

      if (!isNil(onBlur)) {
        onBlur(event, this.props);
      }
    };

    this.handleKeyDown = event => {
      const {
        onKeyDown
      } = this.props; // Since we fully control the open / close of the <Search /> component, we must handle close on "esc" ourself.

      if (event.keyCode === KEYS.esc) {
        this.close(event);
      }

      if (!isNil(onKeyDown)) {
        onKeyDown(event, this.props);
      }
    };

    this.handleOutsideClick = event => {
      const {
        onOutsideClick,
        closeOnOutsideClick
      } = this.props;

      if (closeOnOutsideClick) {
        this.close(event);
      }

      if (!isNil(onOutsideClick)) {
        onOutsideClick(event, this.props);
      }
    };
  }

  static getDerivedStateFromProps(props, state) {
    return getAutoControlledStateFromProps(props, state, SearchInput.autoControlledProps);
  }

  componentDidUpdate() {
    const {
      closeOnBlur,
      closeOnOutsideClick
    } = this.props;

    if (closeOnBlur && closeOnOutsideClick) {
      throw new ArgumentError("SearchInput - The \"closeOnBlur\" and \"closeOnOutsideClick\" props cannot be both \"true\".");
    }
  } // TODO: memoizing search result could greatly improved the performance of this component:
  //  - The shallow comparison done by search-input-controller would not force a re-render when this is the same results


  open(event) {
    const {
      onVisibilityChange
    } = this.props;
    const {
      open
    } = this.state;

    if (!open) {
      this.trySetAutoControlledStateValue({
        open: true
      });

      if (!isNil(onVisibilityChange)) {
        onVisibilityChange(event, true, this.props);
      }
    }
  }

  close(event) {
    const {
      onVisibilityChange
    } = this.props;
    const {
      open
    } = this.state;

    if (open) {
      this.trySetAutoControlledStateValue({
        open: false
      });

      if (!isNil(onVisibilityChange)) {
        onVisibilityChange(event, false, this.props);
      }
    }
  }

  render() {
    const {
      value,
      defaultValue,
      resultRenderer,
      clearOnSelect,
      noResultsMessage,
      minCharacters,
      debounceDelay,
      placeholder,
      disabled,
      autofocus,
      autofocusDelay,
      className
    } = this.props;
    const {
      open,
      visibleResults
    } = this.state;
    return React.createElement(SearchInputController, {
      open: open,
      results: visibleResults,
      value: value,
      defaultValue: defaultValue,
      onValueChange: this.handleValueChange,
      onSearch: this.handleSearch,
      onClear: this.handleClear,
      onBlur: this.handleBlur,
      onKeyDown: this.handleKeyDown,
      onOutsideClick: this.handleOutsideClick,
      resultRenderer: resultRenderer,
      clearOnSelect: clearOnSelect,
      noResultsMessage: noResultsMessage,
      minCharacters: minCharacters,
      debounceDelay: debounceDelay,
      placeholder: placeholder,
      disabled: disabled,
      autofocus: autofocus,
      autofocusDelay: autofocusDelay,
      className: className
    });
  }

}
SearchInput.propTypes = {
  results: arrayOf(shape(RESULT_SHAPE)).isRequired,
  value: string,
  defaultValue: string,
  onValueChange: func.isRequired,
  onVisibilityChange: func,
  onSearch: func,
  onClear: func,
  onBlur: func,
  onOutsideClick: func,
  resultRenderer: func,
  clearOnSelect: bool,
  noResultsMessage: string,
  debounceDelay: number,
  minCharacters: number,
  placeholder: string,
  defaultOpen: bool,
  open: bool,
  disabled: bool,
  autofocus: bool,
  autofocusDelay: number,
  closeOnBlur: bool,
  closeOnOutsideClick: bool,
  className: string
};
SearchInput.defaultProps = {
  onSearch: startsWithSearch,
  minCharacters: 1,
  closeOnBlur: true,
  closeOnOutsideClick: false
};
SearchInput.autoControlledProps = ["open"];